﻿local Engine, Api, Locale, Settings = unpack(Immersive)

-- Constants
local CHATFRAME_NAME = "ChatFrame%d%s"

local GEM_WIDTH = 25
local GEM_HEIGHT = 4
local GEM_ANCHOR = "BOTTOMLEFT"
local GEM_MARGIN = 3
local GEM_GROW_AXIS = "X"
local GEM_GROW_DIRECTION = 1

local CHAT_MSG_MAP = {
	["CHAT_MSG_SAY"] = "SAY",
	["CHAT_MSG_YELL"] = "YELL",
	["CHAT_MSG_BN_WHISPER"] = "BN_WHISPER",
	["CHAT_MSG_WHISPER"] = "WHISPER",
	["CHAT_MSG_GUILD"] = "GUILD",
	["CHAT_MSG_OFFICER"] = "OFFICER",
	["CHAT_MSG_PARTY"] = "PARTY",
	["CHAT_MSG_PARTY_LEADER"] = "PARTY_LEADER",
	["CHAT_MSG_RAID"] = "RAID",
	["CHAT_MSG_RAID_LEADER"] = "RAID_LEADER",
	["CHAT_MSG_RAID_WARNING"] = "RAID_WARNING",
	["CHAT_MSG_INSTANCE_CHAT"] = "INSTANCE_CHAT",
	["CHAT_MSG_INSTANCE_CHAT_LEADER"] = "INSTANCE_CHAT_LEADER"
}

local Default = Engine:GetModule("Default")
local Gems = {}
local IsLayoutRequired = true
local IsOpenRequested = false

local Element = nil
local Background = nil
local Controller = nil

-- Registers the element in standard mode.
function Default:RegisterChatFrameGemsMode()
	local settings = Settings:Get(self.Name).Elements["ChatFrame"]
	Element = Api:RegisterElement("ChatFrame", settings)

	-- Add frames
	for i = 1, NUM_CHAT_WINDOWS do
		for _, type in pairs({ "", "Tab"}) do
			Element:AddFrame(_G[string.format(CHATFRAME_NAME, i, type)])
		end
	end
	Element:AddFrame(FriendsMicroButton)
	Element:AddFrame(ChatFrameMenuButton)
	
	-- Add conditions
	Element:AddConditionRange(settings.Conditions)
	Element:AddCondition(function() return ChatFrame1EditBox:IsShown() end)
	Element:AddCondition(function() return IsOpenRequested end)
	
	Element.OnFadeIn = function(self, frame, alpha)
		if alpha > 0 and frame.wasShown then
			frame.Show = frame.wasShown
			frame.wasShown = nil
			frame:Show()
		end
		if alpha == 1 then
			IsOpenRequested = false
		end
	end
	Element.OnFadeOut = function(self, frame, alpha)
		if alpha == 0 and frame:IsShown() then
			frame.wasShown = frame.Show
			frame.Show = function() end
			frame:Hide()
		end
	end

	Background = CreateFrame("Frame", string.format("%s%sGemBackground", Engine.AddOnName, self.Name), UIParent)
	Background:SetPoint(GEM_ANCHOR, UIParent, GEM_ANCHOR, GEM_WIDTH, 0)
	Background:SetSize(GEM_WIDTH, GEM_HEIGHT)
	Background:SetScript("OnUpdate", function()
		if IsLayoutRequired then
			-- Update the gems layout
			Default:LayoutGems()
		end
	end)

	Controller = CreateFrame("Button", string.format("%s%sGemButton", Engine.AddOnName, self.Name), UIParent)
	Controller:SetSize(GEM_WIDTH, GEM_HEIGHT)
	Controller:SetPoint(GEM_ANCHOR, Background, GEM_ANCHOR, GEM_MARGIN, 0)
	Controller:EnableMouse(true)

	Controller.Texture = Controller:CreateTexture(nil, "ARTWORK")
	Controller.Texture:SetTexture(1, 1, 0, 0.4)
	Controller.Texture:SetSize(GEM_WIDTH, GEM_HEIGHT)
	Controller.Texture:SetPoint("TOPLEFT")

	Controller:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self)
		GameTooltip:SetText(Locale.DEFAULT_MODE_GEMS_CONTROLLER_TOOLTIP)
        GameTooltip:Show()
	end)
	Controller:SetScript("OnLeave", function(self)
		GameTooltip_Hide()
	end)
	Controller:SetScript("OnClick", function(self)
		IsOpenRequested = true
	end)

	-- Add hotspots
	self:RegisterChatFrameHotspots(Element)

	self:RegisterEvent("CHAT_MSG_BN_WHISPER", "OnGemMessage")
	self:RegisterEvent("CHAT_MSG_WHISPER", "OnGemMessage")
	self:RegisterEvent("CHAT_MSG_SAY", "OnGemMessage")
	self:RegisterEvent("CHAT_MSG_YELL", "OnGemMessage")
	self:RegisterEvent("CHAT_MSG_GUILD", "OnGemMessage")
	self:RegisterEvent("CHAT_MSG_OFFICER", "OnGemMessage")
	self:RegisterEvent("CHAT_MSG_PARTY", "OnGemMessage")
	self:RegisterEvent("CHAT_MSG_PARTY_LEADER", "OnGemMessage")
	self:RegisterEvent("CHAT_MSG_RAID", "OnGemMessage")
	self:RegisterEvent("CHAT_MSG_RAID_LEADER", "OnGemMessage")
	self:RegisterEvent("CHAT_MSG_RAID_WARNING", "OnGemMessage")
	self:RegisterEvent("CHAT_MSG_INSTANCE_CHAT", "OnGemMessage")
	self:RegisterEvent("CHAT_MSG_INSTANCE_CHAT_LEADER", "OnGemMessage")
	--self:RegisterEvent("CHAT_MSG_CHANNEL", "OnGemMessage")
end

-- Creates a gem which represents a chat type / channel.
function Default:UnregisterChatFrameGemsMode()
end

-- Chat message handler.
-- message - The message thats received (string)
-- sender - The sender's username. (string)
-- language - The language the message is in. (string)
-- channelString - The full name of the channel, including number. (string)
-- target - The username of the target of the action. Not used by all events. (string)
-- flags - The various chat flags. Like, DND or AFK. (string)
-- unknown1 - This variable has an unknown purpose, although it may be some sort of internal channel id. That however is not confirmed. (number)
-- channelNumber - The numeric ID of the channel. (number)
-- channelName - The full name of the channel, does not include the number. (string)
-- unknown2 - This variable has an unknown purpose although it always seems to be 0. (number)
-- counter - This variable appears to be a counter of chat events that the client recieves. (number)
-- guid - GUID of the person who sent this message. (string)
function Default:OnGemMessage(event, message, sender, language, channelString, target, flags, unknown1, channelNumber, channelName, unknown2, counter, guid)
	if ChatFrame1:GetAlpha() < 0.5 then
		local channel = self:MapChannel(event, channelString)
		if channel ~= nil then
			local gem = Gems[channel]
			if gem == nil then
				gem = self:CreateGem(channel)
				IsLayoutRequired = true
			end
			gem:Increment()
		end
	end
end

-- Creates a channel gem to notify the player of a pending chat message.
function Default:CreateGem(channel)
	local gem  = {}
	gem.Channel = channel
	gem.Counter = 0
	gem.R, gem.G, gem.B = GetMessageTypeColor(channel)

	gem.IsActive = function(self)
		return self.Counter > 0
	end
	gem.Increment = function(self)
		self.Counter = self.Counter + 1
	end
	gem.Reset = function(self)
		self.Counter = 0
	end

	gem.Component = CreateFrame("Button", string.format("%s%sGemButton", Engine.AddOnName, self.Name), UIParent)

	local component = gem.Component
	component:SetSize(GEM_WIDTH, GEM_HEIGHT)
	component:SetPoint(GEM_ANCHOR, Background, GEM_ANCHOR, GEM_MARGIN)
	component:EnableMouse(true)

	component.Texture = component:CreateTexture(nil, "ARTWORK")
	component.Texture:SetTexture(gem.R, gem.G, gem.B, 0.4)
	component.Texture:SetSize(GEM_WIDTH, GEM_HEIGHT)
	component.Texture:SetPoint("TOPLEFT")

	component:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self)
		GameTooltip:SetText(Locale[gem.Channel] or gem.Channel)
		if gem.Counter > 0 then
			GameTooltip:AddLine(string.format(Locale.DEFAULT_MODE_GEMS_CHANNEL_TOOLTIP_UNREAD, gem.Counter), 1, 1, 1, true)
		else
			GameTooltip:AddLine(Locale.DEFAULT_MODE_GEMS_CHANNEL_TOOLTIP_READ, 1, 1, 1, true)
		end
        GameTooltip:Show()
	end)
	component:SetScript("OnLeave", function(self)
		GameTooltip_Hide()
	end)
	component:SetScript("OnClick", function(self)
		for name, gem in pairs(Gems) do
			gem:Reset()
		end
		IsOpenRequested = true
		IsLayoutRequired = true
	end)

	Gems[channel] = gem
	
	return gem
end

-- Updates and renders the gems to the screen.
function Default:LayoutGems()
	IsLayoutRequired = false
	local x, y, count = GEM_MARGIN, 0, 0
	for name, gem in pairs(Gems) do
		local component = gem.Component
		if gem:IsActive() then
			component:Show()
			component:SetPoint(GEM_ANCHOR, Background, GEM_ANCHOR, x, y)
			x, y = self:LayoutGemsGrow(x, y)
			count = count + 1
		else
			component:Hide()
			component:SetPoint(GEM_ANCHOR, Background, GEM_ANCHOR, 0, 0)
		end
	end
	if count == 0 then
		count = 1
		Controller:Show()
	else
		Controller:Hide()
	end
end

-- Grows the x, y positions based on user settings
function Default:LayoutGemsGrow(x, y)
	if GEM_GROW_AXIS == "X" then
		x = x + ((GEM_WIDTH + GEM_MARGIN) * GEM_GROW_DIRECTION)
	else
		y = y + ((GEM_WIDTH + GEM_MARGIN) * GEM_GROW_DIRECTION)
	end
	return x, y
end

function Default:MapChannel(event, channel)
	local map = CHAT_MSG_MAP[event]
	if map == nil then
		return channel
	end
	return map
end