﻿local Engine, Api, Locale, Settings = unpack(Immersive)

-- Extensions
local debug = Engine.Core.Debug

-- Constants
local X_CONTROL_LEFT_COLUMN = -2
local X_CONTROL_RIGHT_COLUMN = 289
local Y_CONTROL_OFFSET = 20
local MIN_ANIM_SECONDS = 0
local MAX_ANIM_SECONDS = 60

local CONTROL_TITLES = {
	BuffFrame = Locale.DEFAULT_BUFF_FRAME_TITLE,
	ChatFrame = Locale.DEFAULT_CHAT_FRAME_TITLE,
	MainMenuBar = Locale.DEFAULT_MAIN_MENU_BAR_TITLE,
	MinimapCluster = Locale.DEFAULT_MINIMAP_CLUSTER_TITLE,
	MultibarBottomLeft = Locale.DEFAULT_MULTIBAR_BOTTOM_LEFT_TITLE,
	MultibarBottomRight = Locale.DEFAULT_MULTIBAR_BOTTOM_RIGHT_TITLE,
	MultibarLeft = Locale.DEFAULT_MULTIBAR_LEFT_TITLE,
	MultibarRight = Locale.DEFAULT_MULTIBAR_RIGHT_TITLE,
	ObjectiveTrackerFrame = Locale.DEFAULT_OBJECTIVE_TRACKER_FRAME_TITLE,
	PartyMemberFrame = Locale.DEFAULT_PARTY_MEMBER_FRAME_TITLE,
	PetActionBarFrame = Locale.DEFAULT_PET_ACTION_BAR_FRAME_TITLE,
	PlayerFrame = Locale.DEFAULT_PLAYER_FRAME_TITLE,
	CompactRaidFrame = Locale.DEFAULT_COMPACT_RAID_FRAME_TITLE,
	StanceBarFrame = Locale.DEFAULT_STANCE_BAR_FRAME_TITLE,
	TargetFrame = Locale.DEFAULT_TARGET_FRAME_TITLE
}

local Default = Engine:GetModule("Default")
local Handlers = {}

-- Registers the Default options panel
function Default:OnRegisterOptions()
	local settings = Settings:Get(self.Name)
	local panel = CreateFrame("Frame", string.format("%s%sOptionsFrame", Engine.AddOnName, self.Name))
	
	panel.name = Locale.CONFIG_MODULE_DEFAULT_NAME
	panel.parent = Locale.CONFIG_MODULE_IMMERSIVE_NAME

	panel.okay = Default.OnOkay
	panel.cancel = Default.OnCancel
	panel.default = Default.OnDefault
	panel.refresh = Default.OnRefresh

	-- Description
	local title = panel:CreateFontString("Title", "ARTWORK", "GameFontNormalLarge")
	title:SetJustifyH("LEFT")
	title:SetJustifyV("TOP")
	title:SetPoint("TOPLEFT", panel, "TOPLEFT", 16, -16)
	title:SetText(Locale.CONFIG_MODULE_DEFAULT_TITLE)

	local subText = panel:CreateFontString("Title", "ARTWORK", "GameFontHighlightSmall")
	subText:SetJustifyH("LEFT")
	subText:SetJustifyV("TOP")
	subText:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
	subText:SetPoint("RIGHT", panel, "RIGHT", -32, 0)
	subText:SetText(Locale.CONFIG_MODULE_DEFAULT_SUBTEXT)

	-- Controls: Enable
	local enabled = CreateFrame("CheckButton", string.format("%s%sEnableCheckButton", Engine.AddOnName, self.Name), panel, "InterfaceOptionsCheckButtonTemplate")
	enabled:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", -2, -8)
	enabled:SetChecked(settings.Enabled)
	enabled:SetScript("OnClick", function(self)
		settings.Enabled = self:GetChecked()
	end)
	enabled:SetScript("OnEnter", function(self)
		if self.Tooltip then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
			GameTooltip:SetText(self.Tooltip, nil, nil, nil, nil, true);
		end
	end)
	enabled.Text:SetText(Locale.CONFIG_MODULE_DEFAULT_ENABLED_TEXT)
	enabled.Tooltip = Locale.CONFIG_MODULE_DEFAULT_ENABLED_TOOLTIP

	-- Controls: ScrollFrame
	scrollframe = CreateFrame("ScrollFrame", nil, panel)
	scrollframe:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", 0, -40)
	scrollframe:SetPoint("BOTTOMRIGHT", -10, 10)
	scrollframe:EnableMouseWheel(true)

	-- Controls: ScrollBar 
	scrollbar = CreateFrame("Slider", nil, scrollframe, "UIPanelScrollBarTemplate")
	scrollbar:SetPoint("TOPRIGHT", panel, "TOPRIGHT", -8, -24)
	scrollbar:SetPoint("BOTTOMRIGHT", panel, "BOTTOMRIGHT", -8, 24)
	scrollbar:SetValueStep(1)
	scrollbar.scrollStep = 40
	scrollbar:SetWidth(16)
	scrollbar:SetScript("OnValueChanged", function(self, value)
		self:GetParent():SetVerticalScroll(value)
	end)

	-- Controls: Element Container
	local container = CreateFrame("Frame", nil, scrollframe)
	container:SetPoint("TOPLEFT", scrollframe, "TOPLEFT")
	container:SetPoint("BOTTOMRIGHT", scrollframe, "BOTTOMRIGHT")

	scrollframe:SetScrollChild(container)
	scrollframe:SetScript("OnMouseWheel", function(self, delta)
		local min, max = scrollbar:GetMinMaxValues()
		local value = math.max(0, math.min(self:GetVerticalScroll() + (delta * scrollbar.scrollStep * -1), max))
		self:SetVerticalScroll(value)
		scrollbar:SetValue(value)
	end)
	
	self:OnRegisterElementOptions(container)

	scrollbar:SetMinMaxValues(0, container:GetHeight())
	scrollbar:SetValue(0)

	InterfaceOptions_AddCategory(panel)
end

-- This method will run when the player clicks "okay" in the Interface
-- Options.
function Default:OnOkay()
	-- Apply changes to elements
	for i = 1, #Handlers do
		Handlers[i]:OnOkay()
	end

	-- Confirm with player to reload the UI
	StaticPopup_Show("IMMERSIVE_RELOADUI_CONFIRM")
end

-- This method will run when the player clicks "cancel" in the Interface
-- Options. Use this to revert their changes.
function Default:OnCancel()
end

-- This method will run when the player clicks "defaults". Use this to
-- revert their changes to your defaults.
function Default:OnDefault()
end

-- This method will run when the Interface Options frame calls its OnShow
-- function and after defaults have been applied via the panel.default
-- method described above. Use this to refresh your panel's UI in case
-- settings were changed without player interaction.
function Default:OnRefresh()
	for i = 1, #Handlers do
		Handlers[i]:OnRefresh()
	end
end

-- Registers the Default options panel
function Default:OnRegisterElementOptions(parent)
	local settings = Settings:Get(self.Name)
	local y = 0

	local names = {}
	for name, _ in pairs(settings.Elements) do
		table.insert(names, name)
	end
	table.sort(names, function(a, b) return a < b end)

	-- Controls: Elements
	for _, element in ipairs(names) do
		local options = settings.Elements[element]

		-- Controls: Enable
		local control = CreateFrame("CheckButton", string.format("%s%s%sEnableCheckButton", Engine.AddOnName, self.Name, element), parent, "InterfaceOptionsCheckButtonTemplate")
		control:SetPoint("TOPLEFT", parent, "TOPLEFT", X_CONTROL_LEFT_COLUMN, -y)
		control:SetChecked(options.Enabled)
		control.Text:SetText(CONTROL_TITLES[element])
		control.Tooltip = string.format(Locale.CONFIG_MODULE_DEFAULT_ELEMENT_ENABLED_TOOLTIP, CONTROL_TITLES[element])
		control:SetScript("OnEnter", function(self)
			if self.Tooltip then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
				GameTooltip:SetText(self.Tooltip, nil, nil, nil, nil, true);
			end
		end)
		control.OnOkay = function(self)
			options.Enabled = self:GetChecked()
		end
		control.OnRefresh = function(self)
			self:SetChecked(options.Enabled)
		end
		self:AddHandler(control)

		-- Controls: Delay Time
		local delay = CreateFrame("Slider", string.format("%s%s%sDelaySlider", Engine.AddOnName, self.Name, element), parent, "OptionsSliderTemplate")
		delay:SetPoint("TOPLEFT", control, "BOTTOMLEFT", 24, -15)
		delay:SetMinMaxValues(MIN_ANIM_SECONDS, MAX_ANIM_SECONDS)
		delay:SetValue(options.DelayTime)
		delay:SetScript("OnValueChanged", function(self)
			self.Text:SetText(string.format(Locale.CONFIG_SLIDER_FORMATTED_TEXT, Locale.CONFIG_SLIDER_TYPE_DELAY_TEXT, math.floor(self:GetValue())))
		end)
		delay.Text = _G[delay:GetName().."Text"]
		delay.Text:SetText(string.format(Locale.CONFIG_SLIDER_FORMATTED_TEXT, Locale.CONFIG_SLIDER_TYPE_DELAY_TEXT, math.floor(delay:GetValue())))
		delay.tooltipText = nil
		delay.tooltipRequirement = nil
		delay.OnOkay = function(self)
			options.DelayTime = math.floor(self:GetValue())
			Api.Elements[element].DelayTime = options.DelayTime
			Api.Elements[element]:Apply()
		end
		delay.OnRefresh = function(self)
			self:SetValue(options.DelayTime)
		end
		self:AddHandler(delay)
		self:AddDependency(control, delay)

		-- Controls: Fade Time
		local fade = CreateFrame("Slider", string.format("%s%s%sFadeSlider", Engine.AddOnName, self.Name, element), parent, "OptionsSliderTemplate")
		fade:SetPoint("TOPLEFT", control, "BOTTOMLEFT", 24, -50)
		fade:SetMinMaxValues(MIN_ANIM_SECONDS, MAX_ANIM_SECONDS)
		fade:SetValue(options.FadeTime)
		fade:SetScript("OnValueChanged", function(self)
			self.Text:SetText(string.format(Locale.CONFIG_SLIDER_FORMATTED_TEXT, Locale.CONFIG_SLIDER_TYPE_FADE_TEXT, math.floor(self:GetValue())))
		end)
		fade.Text = _G[fade:GetName().."Text"]
		fade.Text:SetText(string.format(Locale.CONFIG_SLIDER_FORMATTED_TEXT, Locale.CONFIG_SLIDER_TYPE_FADE_TEXT, math.floor(fade:GetValue())))
		fade.tooltipText = nil
		fade.tooltipRequirement = nil
		fade.OnOkay = function(self)
			options.FadeTime = math.floor(self:GetValue())
			Api.Elements[element].FadeTime = options.FadeTime
			Api.Elements[element]:Apply()
		end
		fade.OnRefresh = function(self)
			self:SetValue(options.FadeTime)
		end
		self:AddHandler(fade)
		self:AddDependency(control, fade)

		-- Controls: Mode
		local mode = CreateFrame("Button", string.format("%s%s%sModeDropDown", Engine.AddOnName, self.Name, element), parent, "UIDropDownMenuTemplate")
		mode:SetPoint("TOPLEFT", control, "BOTTOMLEFT", 4, -100)
		mode:EnableMouse(true)
		mode:SetScript("OnEnter", function(self)
			if self.Tooltip then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
				GameTooltip:SetText(self.Tooltip, nil, nil, nil, nil, true);
			end
		end)
		mode.Label = mode:CreateFontString("Label", "BACKGROUND", "GameFontHighlight")
		mode.Label:SetPoint("BOTTOMLEFT", mode, "TOPLEFT", 16, 3)
		mode.Label:SetText(Locale.CONFIG_MODULE_DEFAULT_MODE_TITLE)
		mode.Tooltip = Locale.CONFIG_MODULE_DEFAULT_MODE_TOOLTIP
		UIDropDownMenu_Initialize(mode, function(self, level)
			for k, v in ipairs(Api:GetAvailableModes(Api.Elements[element])) do
				local info = UIDropDownMenu_CreateInfo()
				info.text = v.Title
				info.value = v.Name
				info.func = function(self)
					UIDropDownMenu_SetSelectedValue(mode, self.value)
				end
				UIDropDownMenu_AddButton(info, level)
			end
		end)
		UIDropDownMenu_SetWidth(mode, 150)
		UIDropDownMenu_SetButtonWidth(mode, 174)
		UIDropDownMenu_SetSelectedValue(mode, options.Mode)
		UIDropDownMenu_SetText(mode, Api.Modes[options.Mode].Title)
		UIDropDownMenu_JustifyText(mode, "LEFT")
		mode.OnOkay = function(self)
			options.Mode = UIDropDownMenu_GetSelectedValue(self)
			Api.Elements[element].Mode = options.Mode
		end
		mode.OnRefresh = function(self)
			UIDropDownMenu_SetSelectedValue(self, options.Mode)
			UIDropDownMenu_SetText(self, Api.Modes[options.Mode].Title)
		end
		self:AddHandler(mode)
		self:AddDependency(control, mode)

		-- Controls: Is Hotspot Active
		local hotspot = CreateFrame("CheckButton", string.format("%s%s%sHotspotCheckButton", Engine.AddOnName, self.Name, element), parent, "InterfaceOptionsCheckButtonTemplate")
		hotspot:SetPoint("TOPLEFT", control, "BOTTOMLEFT", 18, -135)
		hotspot:SetChecked(options.IsHotspot)
		hotspot:SetScript("OnEnter", function(self)
			if self.Tooltip then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
				GameTooltip:SetText(self.Tooltip, nil, nil, nil, nil, true);
			end
		end)
		hotspot.OnOkay = function(self)
			options.IsHotspot = self:GetChecked()
			Api.Elements[element].IsHotspot = options.IsHotspot
		end
		hotspot.OnRefresh = function(self)
			self:SetChecked(options.IsHotspot)
		end
		hotspot.Text:SetText(Locale.CONFIG_MODULE_DEFAULT_HOTSPOT_TEXT)
		hotspot.Tooltip = Locale.CONFIG_MODULE_DEFAULT_HOTSPOT_TOOLTIP
		self:AddHandler(hotspot)
		self:AddDependency(control, hotspot)
		
		-- Controls: Conditions
		for name, condition in pairs(Api.Conditions) do
			local conditional = CreateFrame("CheckButton", string.format("%s%s%s%sConditionCheckButton", Engine.AddOnName, self.Name, element, name), parent, "InterfaceOptionsCheckButtonTemplate")
			conditional:SetPoint("TOPLEFT", parent, "TOPLEFT", X_CONTROL_RIGHT_COLUMN, -y)
			conditional:SetChecked(options.Conditions[name] or false)
			conditional.Text:SetText(condition.Title)
			conditional.Tooltip = condition.Tooltip
			conditional:SetScript("OnEnter", function(self)
				if self.Tooltip then
					GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
					GameTooltip:SetText(self.Tooltip, nil, nil, nil, nil, true);
				end
			end)
			conditional.OnOkay = function(self)
				options.Conditions[name] = self:GetChecked()
			end
			conditional.OnRefresh = function(self)
				self:SetChecked(options.Conditions[name] or false)
			end
			self:AddHandler(conditional)
			self:AddDependency(control, conditional)
			
			y = y + Y_CONTROL_OFFSET
		end
		
		y = y + Y_CONTROL_OFFSET
	end

	parent:SetSize(128, y - (y / #names))
end

-- Adds an element option category to the cache.
function Default:AddHandler(handler)
	table.insert(Handlers, handler)
end

-- Creates a dependency between a control and an enable/disable checkbox.
function Default:AddDependency(parent, child)
	if parent.Children == nil then
		parent.Children = {}
		parent:SetScript("OnClick", function(self)
			for i = 1, #self.Children do
				if self:GetChecked() then
					self.Children[i]:Enable()
				else
					self.Children[i]:Disable()
				end
			end
		end)
	end

	table.insert(parent.Children, child)
	if parent:GetChecked() then
		child:Enable()
	else
		child:Disable()
	end
end

-- Prompt asking the player to reload the UI.
StaticPopupDialogs["IMMERSIVE_RELOADUI_CONFIRM"] = {
	text = Locale.DEFAULT_RELOAD_REQUIRED,
	button1 = Locale.DEFAULT_RELOAD_REQUIRED_YES,
	button2 = Locale.DEFAULT_RELOAD_REQUIRED_NO,
	OnAccept = function()
		ReloadUI()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3
}